/*
Copyright (c) 2000 Scott Wyatt (scwyatt@fedex.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;
import org.gjt.sp.jedit.msg.*;
import org.gjt.sp.util.Log;

public class WheelMousePlugin extends EditPlugin
{
	private Vector windowHWND = new Vector();
	private boolean success = false;


	public void start()
	{
		String dir = jEdit.getJEditHome();
		File dll = new File(MiscUtilities.constructPath(dir, "jars", "MouseWheel.dll"));
		String dllpath = dll.getAbsolutePath();

		if (dll.exists())
			loadLibrary(dllpath);
		else
		{
			String settings = jEdit.getSettingsDirectory();
			if (settings == null)
			{
				Log.log(Log.ERROR, this, "the required native library MouseWheel.dll could not be found; tried " + dllpath);
				return;
			}
			File dll2 = new File(MiscUtilities.constructPath(settings, "jars", "MouseWheel.dll"));
			String dllpath2 = dll2.getAbsolutePath();
			if (dll2.exists())
				loadLibrary(dllpath2);
			else
				Log.log(Log.ERROR, this, "the required native library MouseWheel.dll could not be found; tried " + dllpath + " and " + dllpath2);
		}
	}


	public void createOptionPanes(OptionsDialog optionsDialog)
	{
		if (success)
			optionsDialog.addOptionPane(new WheelMouseOptionPane());
	}


	private void loadLibrary(String filename)
	{
		try
		{
			Log.log(Log.MESSAGE, this, "Loading shared library: " + filename);
			System.load(filename);
			success = true;
			startListeningForOtherWindows();
		}
		catch (UnsatisfiedLinkError ule)
		{
			Log.log(Log.ERROR, this, "could not load required native library: " + filename);
			throw ule;
		}
	}


	private void addWheelMouseListener(Component comp)
	{
		JFrameEx jframeex = new JFrameEx();
		jframeex.setComponent(comp);
		if(windowHWND.indexOf(new Integer(jframeex.getHWND())) == -1)
		{
			jframeex.setHook(jframeex.getHWND());
			windowHWND.addElement(new Integer(jframeex.getHWND()));
		}
		else
			jframeex = null;
	}


	private void removeWheelMouseListener(Component comp)
	{
		JFrameEx jframeex = new JFrameEx();
		jframeex.setComponent(comp);
		jframeex.resetHook(jframeex.getHWND());
		windowHWND.removeElement(new Integer(jframeex.getHWND()));
	}


	private void startListeningForOtherWindows()
	{
		Log.log(Log.MESSAGE, WheelMousePlugin.class, "Adding WheelMouse AWTEventListener.");
		Toolkit.getDefaultToolkit().addAWTEventListener(new WindowAWTEvent(), AWTEvent.WINDOW_EVENT_MASK);
	}


	public class WindowAWTEvent implements AWTEventListener
	{
		public void eventDispatched(AWTEvent awtevt)
		{
			if(awtevt instanceof WindowEvent)
			{
				WindowEvent wevt = (WindowEvent)awtevt;
				if(wevt.getID() == WindowEvent.WINDOW_ACTIVATED)
					addWheelMouseListener(wevt.getComponent());
				if(wevt.getID() == WindowEvent.WINDOW_CLOSING)
					removeWheelMouseListener(wevt.getComponent());
			}
		}
	}
}
